package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.params.analyze.GradeBatchParam;
import com.qmth.distributed.print.business.bean.result.analyze.GradeBatchResult;
import com.qmth.distributed.print.business.entity.GradeBatch;
import com.qmth.distributed.print.business.entity.GradeBatchPaper;
import com.qmth.distributed.print.business.enums.GradeAnalyzePaperStatusEnum;
import com.qmth.distributed.print.business.mapper.GradeBatchMapper;
import com.qmth.distributed.print.business.service.GradeBatchPaperService;
import com.qmth.distributed.print.business.service.GradeBatchService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * 分析-批次表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2022-05-20
 */
@Service
public class GradeBatchServiceImpl extends ServiceImpl<GradeBatchMapper, GradeBatch> implements GradeBatchService {
    @Resource
    GradeBatchPaperService gradeBatchPaperService;

    @Override
    public IPage<GradeBatchResult> gradeBatchPage(String batchName, int pageNumber, int pageSize, SysUser requestUser) {
        batchName = SystemConstant.translateSpecificSign(batchName);
        return this.baseMapper.findGradeBatchPage(new Page<>(pageNumber, pageSize), batchName, requestUser.getSchoolId());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long editGradeBatch(GradeBatchParam gradeBatchParam, SysUser requestUser) {
        Long schoolId = requestUser.getSchoolId();
        Long id = gradeBatchParam.getId();
        String batchName = gradeBatchParam.getBatchName();
        Long batchTime = gradeBatchParam.getBatchTime();

        // 教研分析批次名称学校唯一
        GradeBatch check = this.getOne(new QueryWrapper<GradeBatch>().lambda().eq(GradeBatch::getSchoolId, schoolId).eq(GradeBatch::getBatchName, batchName));
        if (Objects.nonNull(check) && !Objects.equals(check.getId(), id)) {
            throw ExceptionResultEnum.ERROR.exception("已存在名称为： " + batchName + "，的教研分析批次");
        }

        GradeBatch gradeBatch = new GradeBatch();
        gradeBatch.setBatchName(batchName);
        gradeBatch.setBatchTime(batchTime);
        gradeBatch.setBatchCode(DateFormatUtils.format(batchTime, "yyyyMMdd"));
        gradeBatch.setStatus(GradeAnalyzePaperStatusEnum.SETTING_GRADE_PAPER);
        gradeBatch.setSchoolId(schoolId);
        if (!SystemConstant.longNotNull(id)) {
            id = SystemConstant.getDbUuid();
        }
        gradeBatch.setId(id);

        this.saveOrUpdate(gradeBatch);
        return id;
    }

    @Override
    public void deleteGradeBatch(List<Long> idList, SysUser requestUser) {
        Long schoolId = requestUser.getSchoolId();
        if (idList == null || idList.size() == 0) {
            throw ExceptionResultEnum.ERROR.exception("请选择要删除的批次对象");
        }
        // 要删除的批次试卷集合
        List<GradeBatchPaper> gradeBatchPaperList = gradeBatchPaperService.list(new QueryWrapper<GradeBatchPaper>()
                .lambda()
                .eq(GradeBatchPaper::getSchoolId, schoolId)
                .in(GradeBatchPaper::getBatchId, idList));

        if (gradeBatchPaperList != null && gradeBatchPaperList.size() > 0) {
            Set<Long> batchIdSet = gradeBatchPaperList.stream().map(GradeBatchPaper::getBatchId).collect(Collectors.toSet());
            List<String> cantDeleteBatchNameList = this.list(new QueryWrapper<GradeBatch>()
                            .lambda()
                            .in(GradeBatch::getId, batchIdSet))
                    .stream()
                    .map(GradeBatch::getBatchName)
                    .distinct().collect(Collectors.toList());

            throw ExceptionResultEnum.ERROR.exception("成绩分析批次【" + String.join(",", cantDeleteBatchNameList) + "】存在分析课程,不能删除");
        }
        this.removeByIds(idList);
    }
}
